//    ___________     __         .__        
//    \__    ___/____/  |________|__| ______
//      |    |_/ __ \   __\_  __ \  |/  ___/
//      |    |\  ___/|  |  |  | \/  |\___ \ 
//      |____| \___  >__|  |__|  |__/____  >
//                 \/                    \/ 

// What's a game boy without Teris?

// You could play the game in single- or multiplayer mode.
// If you turn the MGK on, it serches 2 seconds for another MGK. If none is find the single player mode starts. If one is found you are in a two player game.

// In single player mode you could press A or B for pause and while the game is paused you could adjust the sound volume with the arrows.
// Your point, cleared rows and the level you are in are displayed on an LCD. If you are using an LCD there will be a highscore saving the best game results.

// In multiplayer mode there is no pause. If you manage to clear a full row, your opponent will be punnished. The more rows you clear at once the bigger the punnishment.

// Consult the WIKI for instructions on how to build a connection cable for two players or the LCD display.









#include <EEPROM.h>
#include <gamekit_2_1.h>
#include <avr/pgmspace.h>

// pins that are connected to LCD
int Dout = 6;
int STR = 7;
int CLK = 5;

// Communication in two-player-mode
int serialdata;
boolean SinglePlayer = true;

long timer;
long Millis_per_Step=1000;
int Wait_On_Top = 0;
int Wait_On_Top_Limit = 3;

char Field[5][8];

int Stone_Nr = 5;
int Rotation = 0;
int stone_pos_x = 1;
int stone_pos_y = 0;
int stone_color = 15;   //Which is the current color
int color_normal = 15;
int color_top = 20;
int color_fixed = 7;

int Points = 0;
int Rows = 0;
int Level = 1;
int Rows_Per_Level = 6;

prog_char Stones[6][4][9] PROGMEM = {{ {1,1,0, 1,0,0, 0,0,0} ,
                                       {1,1,0, 0,1,0, 0,0,0} ,
                                       {0,1,0, 1,1,0, 0,0,0} ,
                                       {1,0,0, 1,1,0, 0,0,0} },
                                       
                                     { {1,0,0, 0,1,0, 0,0,0} ,
                                       {0,1,0, 1,0,0, 0,0,0} ,
                                       {1,0,0, 0,1,0, 0,0,0} ,
                                       {0,1,0, 1,0,0, 0,0,0} },
                                       
                                     { {1,1,0, 1,1,0, 0,0,0} ,
                                       {1,1,0, 1,1,0, 0,0,0} ,
                                       {1,1,0, 1,1,0, 0,0,0} ,
                                       {1,1,0, 1,1,0, 0,0,0} },
                                       
                                     { {1,1,0, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,1,0, 0,0,0} ,
                                       {1,1,0, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,1,0, 0,0,0} },
                                       
                                     { {0,1,0, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,0,0, 0,0,0} },
                                       
                                     { {0,1,0, 0,1,0, 0,1,0} ,
                                       {1,1,1, 0,0,0, 0,0,0} ,
                                       {0,1,0, 0,1,0, 0,1,0} ,
                                       {1,1,1, 0,0,0, 0,0,0} }};

// Hitlist
char Name_HL[3][5];
int Points_HL[3];
int Level_HL[3];

void setup(){

  LcdInit();
  
  gamekit.Begin();
  Serial.begin(9600);
  for (int Counter=0; Counter<5; Counter++) {
    Field[Counter][7] = 1;
  }
  
  gamekit.set_button_timing(50,10);

}


//---------------------------------------------------
//             Random Number Generator
//---------------------------------------------------
long denominator = 172373;
long absolute    = 572665;
long multiplier =  1146;
long summand     = 568781;
double generate_rand_double() {
    absolute = (multiplier*absolute+summand+(10*gamekit.get_systemcounter()%denominator)+millis() )%denominator;
    return float(absolute)/denominator;
}






//---------------------------------------------------
//                 Input / Output
//---------------------------------------------------
boolean Butt_Right_Pressed = false;
long millis_last_down_move = 0;

void ReadKeys() {
  if(gamekit.button_pressed(butt_RIGHT)) {
    absolute = absolute + 5000;    //Random generator
    RotateStone();
  }
  if(gamekit.button_pressed(butt_DOWN)) {
    absolute = absolute + 10000;    //Random generator
    Move_Stone_One_Right();
  }
  if(gamekit.button_pressed(butt_UP)) {
    absolute = absolute - 1000;     //Random generator
    Move_Stone_One_Left();
  }  
  if( (int)gamekit.get_buttons() == 47 ) {    // Down movement
    Butt_Right_Pressed = true;
  } else {
    Butt_Right_Pressed = false;
  }  
  
  if ( (Butt_Right_Pressed) && (millis() > millis_last_down_move+100) ) {
    millis_last_down_move = millis();
    absolute = absolute + 50000;    //Random generator
    ClearStone();
    Move_Stone_One_Down();
    PaintStone(stone_color);
  } 
  if (  (gamekit.button_pressed(butt_FUNCA)) || (gamekit.button_pressed(butt_FUNCB))  ) {
    if (SinglePlayer) {
      Menu();    //Pause is disabled in multi player mode
    }
  }
}

void PutPixel(int x_, int y_, int color) {
  gamekit.set_pixel(x_, 6-y_, color);
}

void PaintStone(int color) {
  for (int Counter=0; Counter<9; Counter++) {
    if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter]) == 1) {
      PutPixel(stone_pos_x+Counter%3, stone_pos_y+Counter/3, color);
    }
  } 
}

void PaintRows(int first_row, int nr_of_rows, int color) {
  for (int Counter_row=0; Counter_row<nr_of_rows; Counter_row++) {
    for (int Counter_x=0; Counter_x<5; Counter_x++) {
      PutPixel(Counter_x, first_row+Counter_row, color);
    }
  }
}

void MoveRowsOneDown(int Bottom_Row) {
  for (int Counter_row=Bottom_Row; Counter_row>=0; Counter_row--) {
    for (int Counter_x=0; Counter_x<5; Counter_x++) {
      Field[Counter_x][Counter_row+1] = Field[Counter_x][Counter_row];
      PutPixel(Counter_x, Counter_row+1, color_fixed*Field[Counter_x][Counter_row+1]);
    }
  }
  for (int Counter_x=0; Counter_x<5; Counter_x++) {    //Delete upper row
    Field[Counter_x][0] = 0;
    PutPixel(Counter_x, 0, 0);
  }
}

void ClearStone() {
  for (int Counter_y=0; Counter_y<3; Counter_y++) {
    for (int Counter_x=0; Counter_x<3; Counter_x++) {
      if (Field[stone_pos_x+Counter_x][stone_pos_y+Counter_y]==1) { 
        PutPixel(stone_pos_x+Counter_x, stone_pos_y+Counter_y, color_fixed);
      } else {
        PutPixel(stone_pos_x+Counter_x, stone_pos_y+Counter_y, 0);
      }
    }
  }
}

void WriteField() {
  for (int Counter_y=0; Counter_y<8; Counter_y++) {
    for (int Counter_x=0; Counter_x<5; Counter_x++) {
      Serial.print((int)Field[Counter_x][Counter_y]); Serial.print(" ");
    }  
    Serial.println("");
  }
  Serial.println("---------");
}

void LCD_Write_Points() {
  Set_LCD_Cursur(0, 0); LcdDataWrite('P'); LcdDataWrite('o'); LcdDataWrite('i'); LcdDataWrite('n'); LcdDataWrite('t'); LcdDataWrite('s'); LcdDataWrite(':'); LcdDataWrite(' '); LcdDataWrite(' '); LcdDataWrite(' '); LcdDataWrite(' '); LcdNumberWrite(Points,5);
  Set_LCD_Cursur(1, 0); LcdDataWrite('R'); LcdDataWrite('o'); LcdDataWrite('w'); LcdDataWrite('s'); LcdDataWrite(':'); LcdNumberWrite(Rows,3);
                        LcdDataWrite(' '); LcdDataWrite(' '); LcdDataWrite('L'); LcdDataWrite('v'); LcdDataWrite('L'); LcdDataWrite(':'); LcdNumberWrite(Level,2);
}





//---------------------------------------------------
//                       Melody
//---------------------------------------------------
//Tetris
prog_char melody_1[110][2] PROGMEM  = {{2,17},{1,12},{1,13},{2,15},{1,13},{1,12}  ,{2,10},{1,10},{1,13},{2,17},{1,15},{1,13}, {3,12},{1,13},{2,15},{2,17} ,{2,13},{6,10} ,{1,0},{2,15},{1,18},{2,22},{1,20},{1,18} ,{3,17},{1,13},{2,17},{1,15},{1,13} ,{2,12},{1,12},{1,13},{2,15},{2,17}  ,{2,13},{2,10},{2,10},{2,0},    {2,17},{1,12},{1,13},{2,15},{1,13},{1,12}  ,{2,10},{1,10},{1,13},{2,17},{1,15},{1,13}, {3,12},{1,13},{2,15},{2,17} ,{2,13},{2,10},{2,10},{2,0} ,{1,0},{2,15},{1,18},{2,22},{1,20},{1,18} ,{3,17},{1,13},{2,17},{1,15},{1,13} ,{2,12},{1,12},{1,13},{2,15},{2,17}  ,{2,13},{2,10},{2,10},{2,0} ,{2,5},{2,5},{2,1},{2,1} ,{2,3},{2,3},{2,-1},{2,-1} ,{2,1},{2,1},{2,-3},{2,-3}, {2,-4},{2,-4},{2,-1},{2,0}, {2,5},{2,5},{2,1},{2,1}, {2,3},{2,3},{2,-1},{2,-1}, {2,1},{2,5},{2,10},{2,10}, {2,9},{2,9},{2,0},{2,0} };
int melody_length = 111;
int melody_position = melody_length;
int melody_speed = 250;
int sound_volume;
long last_melody_tone_played_at_time = 0;


void Play_Melody() {
  if ( (millis() >= last_melody_tone_played_at_time + melody_speed*(char)pgm_read_byte(&melody_1[melody_position][0]))  && (SinglePlayer) ) {
    last_melody_tone_played_at_time = millis();
    melody_position++;
    if (melody_position>melody_length) {
      melody_position=0;
    }      
    play_tone((char)pgm_read_byte(&melody_1[melody_position][1]), -1, sound_volume);
  }
}

void play_tone(int distance_to_C_, int duration_, int loudness_){
  if (loudness_!=0) {
    if (distance_to_C_!=0) {
      gamekit.stop_tone();
      delay(2);
      gamekit.start_tone(246  *  pow(float(2), float(distance_to_C_)/12.0));
    } else {
      gamekit.stop_tone();
    }  
  } 
  if (duration_>=0) {
    delay(duration_);
    gamekit.stop_tone();
  } else {
    delay(1);
  }
}

void Pause_Game(int milli_secs) {   //like delay() but does not destroy the melody playback
  long start_time = millis();
  do {
    delay(1);  
    Play_Melody();   
  } while (millis()<start_time+milli_secs);
  if (SinglePlayer==false) {
    Read_Serial_In();
  }
}




//---------------------------------------------------
//                 Stone Controle
//---------------------------------------------------

void Move_Stone_One_Right() {
  if ( (Check_Wall_Right()==false) && (  (Wait_On_Top<Wait_On_Top_Limit) || (Check_Stones_Right()==false)  )  ) {
    ClearStone();
    stone_pos_x++;
    PaintStone(stone_color);
  }
}
  
void Move_Stone_One_Left() {
  //Serial.print((int)Check_Wall_Left());
  //Serial.print((int)Check_Stones_Left());
  if ( (Check_Wall_Left()==false) && (  (Wait_On_Top<Wait_On_Top_Limit) || (Check_Stones_Left()==false)  )  ) {
    ClearStone();
    stone_pos_x--;
    PaintStone(stone_color);
  }
}
  
void Move_Stone_One_Down() {
  Wait_On_Top = Wait_On_Top_Limit;
  stone_color = color_normal;
  if (Check_Ground_Reached(true)) {
    PaintStone(color_fixed);
    Copy_Stone_To_Field();
    //WriteField();
    Check_Full_Row();
    Check_Loose();
    //WriteField();
    CreateNewStone();
    millis_last_down_move = millis() + 500;
  } else {
    ClearStone();
    stone_pos_y++;
    PaintStone(stone_color);
  }
  timer = millis();
}

void CreateNewStone() {  
  Read_Serial_In();
  if ( (serialdata=='c') || (serialdata=='d') || (serialdata=='e') ) {
    stone_pos_y = 0;
    stone_pos_x = (int)(generate_rand_double()*4);
    if (generate_rand_double()<0.5)
      Move_Stone_One_Right();
    stone_color = color_top;
    Rotation = (int)(generate_rand_double()*4);
    if (serialdata=='c') { Stone_Nr = 4; Set_LCD_Cursur(0, 0); LcdDataWrite('c'); }
    if (serialdata=='d') { Stone_Nr = 3; Set_LCD_Cursur(0, 0); LcdDataWrite('d'); }
    if (serialdata=='e') { Stone_Nr = 0; Set_LCD_Cursur(0, 0); LcdDataWrite('e'); }
    PaintStone(stone_color);
    serialdata = -99;
    do {
      delay(150);
      Move_Stone_One_Down();
      //Set_LCD_Cursur(0, 0); LcdNumberWrite(Wait_On_Top);
      //Set_LCD_Cursur(1, 0); LcdNumberWrite(stone_pos_x);LcdNumberWrite(stone_pos_y);
    } while (Wait_On_Top >= Wait_On_Top_Limit);
  } else {
    stone_pos_y = 0;
    stone_pos_x = 1;
    Wait_On_Top = 0;
    stone_color = color_top;
    Stone_Nr = (int)(generate_rand_double()*6);
    Rotation = (int)(generate_rand_double()*4);
    PaintStone(stone_color);
  }
}

void RotateStone() {
  if ( (Check_Rotation_Possible_wall()==true)  &&  ( (Check_Rotation_Possible_stones()==true) || (Wait_On_Top<Wait_On_Top_Limit) ) ) {
    ClearStone();
    Rotation++;
    if (Rotation > 3) {
      Rotation = 0;
    }
    PaintStone(stone_color);
  }
}  
    
boolean Check_Ground_Reached(boolean After_Next_Step) {
  if (After_Next_Step) { stone_pos_y++; }
  for (int Counter=0; Counter<9; Counter++) {
    if ( (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter])==1)  &&  (Field[stone_pos_x+Counter%3][stone_pos_y+Counter/3]==1) ) {
      if (After_Next_Step) { stone_pos_y--; }
      return true;
    }
  }  
  if (After_Next_Step) { stone_pos_y--; }
  return false;
}

boolean Check_Stones_Left() {   //Is another stone in the way?
  stone_pos_x--;
  for (int Counter=0; Counter<9; Counter++) {
    if ( (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter])==1)  &&  (Field[stone_pos_x+Counter%3][stone_pos_y+Counter/3]==1) ) {
      stone_pos_x++;
      return true;
    }
  }  
  stone_pos_x++;
  return false;  
}

boolean Check_Wall_Left() {    //Is the end of the field reached?
  for (int Counter_x=0; Counter_x<3; Counter_x++) {
    for (int Counter_y=0; Counter_y<3; Counter_y++) {
      if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter_y*3+Counter_x])==1) {
        if (stone_pos_x+Counter_x==0) {
          return true;
        }
      }
    }
  }
  return false;  
}

boolean Check_Stones_Right() {   //Is another stone in the way?
  stone_pos_x++;
  for (int Counter=0; Counter<9; Counter++) {
    if ( (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter])==1)  &&  (Field[stone_pos_x+Counter%3][stone_pos_y+Counter/3]==1) ) {
      stone_pos_x--;
      return true;
    }
  }  
  stone_pos_x--;
  return false;  
}

boolean Check_Wall_Right() {    //Is the end of the field reached?
  for (int Counter_x=2; Counter_x>=0; Counter_x--) {
    for (int Counter_y=0; Counter_y<3; Counter_y++) {
      if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter_y*3+Counter_x])==1) {
        if (stone_pos_x+Counter_x==4) {
          return true;
        }
      }
    }
  }
  return false;  
}

boolean Check_Rotation_Possible_stones() {
  Rotation++; if (Rotation >= 4) { Rotation = 0; }
  //Check if other stones are in the way
  for (int Counter=0; Counter<9; Counter++) {
    if ( (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter])==1)  &&  (Field[stone_pos_x+Counter%3][stone_pos_y+Counter/3]==1) ) {
      Rotation--; if (Rotation <= -1) { Rotation = 3; }
      return false;
    }
  }  
  Rotation--; if (Rotation <= -1) { Rotation = 3; }
  return true;
}
  
boolean Check_Rotation_Possible_wall() {
  Rotation++; if (Rotation >= 4) { Rotation = 0; }
  //Check if point is moved out of field (left side)
  for (int Counter_x=0; Counter_x<3; Counter_x++) {
    for (int Counter_y=0; Counter_y<3; Counter_y++) {
      if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter_y*3+Counter_x])==1) {  //most left stone found
        if (stone_pos_x+Counter_x<0) {
          Rotation--; if (Rotation <= -1) { Rotation = 3; }
          return false;
        }
      }
    }
  }
  //Check if point is moved out of field (right side)
  for (int Counter_x=2; Counter_x>=0; Counter_x--) {
    for (int Counter_y=0; Counter_y<3; Counter_y++) {
      if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter_y*3+Counter_x])==1) {  //most left stone found
        if (stone_pos_x+Counter_x>4) {
          Rotation--; if (Rotation <= -1) { Rotation = 3; }
          return false;
        }
      }
    }
  }
  Rotation--; if (Rotation <= -1) { Rotation = 3; }
  return true;
}



//---------------------------------------------------
//                 Game Procedures
//---------------------------------------------------

void Copy_Stone_To_Field() {
  for (int Counter=0; Counter<9; Counter++) {
    if (pgm_read_byte(&Stones[Stone_Nr][Rotation][Counter])==1) {
      //Serial.print("ddd");Serial.print(stone_pos_x);Serial.print(" ");Serial.print(stone_pos_y);
      Field[stone_pos_x+Counter%3][stone_pos_y+Counter/3]=1;
    }
  }  
}

void Check_Full_Row() {
  int Nr_of_full_rows = 0;
  int First_full_row = -1;
  for (int Counter=0; Counter<7; Counter++) {    //find full rows
    if ( (Field[0][Counter]==1) && (Field[1][Counter]==1) && (Field[2][Counter]==1) && (Field[3][Counter]==1) && (Field[4][Counter]==1) ) {    //Full Line
      Nr_of_full_rows++;
      if (First_full_row==-1) {
        First_full_row = Counter;
      }
    }
  }
  if (First_full_row!=-1) {    //Do with full rows:
//    Serial.print(First_full_row); Serial.print(", "); Serial.println(Nr_of_full_rows);
    for (int Counter=0; Counter<2; Counter++) {           //Let them blink
      PaintRows(First_full_row, Nr_of_full_rows, 0);            Pause_Game(Millis_per_Step/2);    
      PaintRows(First_full_row, Nr_of_full_rows, color_fixed);  Pause_Game(Millis_per_Step/2);
    }
    PaintRows(First_full_row, Nr_of_full_rows, 0);              Pause_Game(Millis_per_Step/2);

    if (SinglePlayer) {                                                 //Play a sound
      if (Nr_of_full_rows==3) {
        gamekit.play_tone(1000,200,sound_volume-1); delay(30);
        gamekit.play_tone(1000,200,sound_volume-1); delay(60);
        for (int Counter=1000; Counter>600; Counter=Counter-50) {
          gamekit.play_tone(Counter,50,sound_volume-1);
        }
      }      
    } else {
      switch (Nr_of_full_rows) {
        case 1:
          gamekit.play_tone(1000,250,0);
          Serial.write('c');     //"c" = one line cleared
          break;
        case 2:
          gamekit.play_tone(1000,200,0); delay(30);
          gamekit.play_tone(1000,200,0);
          Serial.write('d');     //"d" = two line cleared
          break;
        case 3:
          gamekit.play_tone(1000,200,0); delay(30);
          gamekit.play_tone(1000,200,0); delay(60);
          for (int Counter=1000; Counter>600; Counter=Counter-50) {
            gamekit.play_tone(Counter,50,0);
          }
          Serial.write('e');     //"e" = three line cleared
          break;
      }
    }

    for (int Counter=0; Counter<Nr_of_full_rows; Counter++) {     //Move rows above down
//      Serial.print(First_full_row+Counter-1); Serial.println("");
      MoveRowsOneDown(First_full_row+Counter-1);
      Pause_Game(Millis_per_Step/3);
    }
    
    if (SinglePlayer) {
      //New level?
      if ((Rows-1)/Rows_Per_Level < (Rows-1+Nr_of_full_rows)/Rows_Per_Level) {
        Level++;
        Millis_per_Step = Millis_per_Step * 85 / 100;
        melody_speed    = melody_speed    * 90 / 100;
      }
    } else {
      Millis_per_Step = 1000 - sqrt((millis()/50));
    }
    
    switch (Nr_of_full_rows) {
      case 1:
        Points = Points+100;
        Rows = Rows+1;
        break;
      case 2:
        Points = Points+225;
        Rows = Rows+2;
        break;
      default: 
        Points = Points+350;
        Rows = Rows+3;
    }

    LCD_Write_Points();
  }
}

void Check_Loose() {
  for (int Counter=0; Counter<5; Counter++) {
    if (Field[Counter][0]==1) {      //Player has lost
      PaintRows(6,1,15);  gamekit.play_tone(700,90,sound_volume-1);
      PaintRows(5,1,15);  gamekit.play_tone(600,90,sound_volume-1);
      PaintRows(4,1,15);  gamekit.play_tone(500,90,sound_volume-1);
      PaintRows(3,1,15);  gamekit.play_tone(400,90,sound_volume-1);
      PaintRows(2,1,15);  gamekit.play_tone(300,90,sound_volume-1);
      PaintRows(1,1,15);  gamekit.play_tone(200,90,sound_volume-1);
      PaintRows(0,1,15);  gamekit.play_tone(100,90,sound_volume-1);
 
      if (SinglePlayer==false) {
        Serial.write('f');     //"f" = I have lost
        Clear_LCD();
        Set_LCD_Cursur(0, 0); LcdDataWrite('Y'); LcdDataWrite('o'); LcdDataWrite('u'); LcdDataWrite(' '); LcdDataWrite('L'); LcdDataWrite('o'); LcdDataWrite('o'); LcdDataWrite('s'); LcdDataWrite('e'); LcdDataWrite('!'); Set_LCD_Cursur(2, 0);
        while (true) {
          delay(100);
        }
      }
 
      Clear_LCD();
      Set_LCD_Cursur(0, 0); LcdDataWrite('G'); LcdDataWrite('a'); LcdDataWrite('m'); LcdDataWrite('e'); LcdDataWrite(' '); LcdDataWrite('O'); LcdDataWrite('v'); LcdDataWrite('e'); LcdDataWrite('r'); LcdDataWrite(' '); LcdDataWrite(' '); LcdDataWrite(' '); LcdDataWrite(':'); LcdDataWrite('-'); LcdDataWrite('('); LcdDataWrite(' ');
      delay(2000);
      Set_LCD_Cursur(0, 12); LcdDataWrite(';'); Set_LCD_Cursur(1, 24); delay(500);      
      Set_LCD_Cursur(0, 12); LcdDataWrite(':'); Set_LCD_Cursur(1, 24); delay(1000);      

      int Reached_Place = 4;
      for (int Counter2=2; Counter2>=0; Counter2--) {
        if (Points > Points_HL[Counter2]) {
          Reached_Place--;
        }
      }
      Clear_LCD();
      if (Reached_Place < 4) {    
        //Free up the right place
        for (int Counter2=1; Counter2>Reached_Place-2; Counter2--) {
          Points_HL[Counter2+1] = Points_HL[Counter2];
          Level_HL[Counter2+1] = Level_HL[Counter2];
          Name_HL[Counter2+1][0] = Name_HL[Counter2][0];
          Name_HL[Counter2+1][1] = Name_HL[Counter2][1];
          Name_HL[Counter2+1][2] = Name_HL[Counter2][2];
          Name_HL[Counter2+1][3] = Name_HL[Counter2][3];
          Name_HL[Counter2+1][4] = Name_HL[Counter2][4];
        }
        
        //Write player into Hitlist
        Set_LCD_Cursur(0, 0); LcdDataWrite('P'); LcdDataWrite('l'); LcdDataWrite('a'); LcdDataWrite('c'); LcdDataWrite('e'); LcdDataWrite(' '); LcdNumberWrite(Reached_Place);  
        Set_LCD_Cursur(1, 0); LcdDataWrite('Y'); LcdDataWrite('o'); LcdDataWrite('u'); LcdDataWrite('r'); LcdDataWrite(' '); LcdDataWrite('N'); LcdDataWrite('a'); LcdDataWrite('m'); LcdDataWrite('e'); LcdDataWrite(':'); LcdDataWrite(' '); LcdDataWrite('A'); LcdDataWrite('a'); LcdDataWrite('a'); LcdDataWrite('a'); LcdDataWrite('a');    
        int Character_Nr = 0;
        Name_HL[Reached_Place-1][0]='A'; Name_HL[Reached_Place-1][1]='a'; Name_HL[Reached_Place-1][2]='a'; Name_HL[Reached_Place-1][3]='a'; Name_HL[Reached_Place-1][4]='a';
        Set_LCD_Cursur(1, 11);
        LcdCommandWrite(0x0F); //  cursor blink
        Points_HL[Reached_Place-1] = Points;
        Level_HL[Reached_Place-1] = Level;
        do {
          if (gamekit.button_pressed(butt_RIGHT)) {
            Name_HL[Reached_Place-1][Character_Nr] = Name_HL[Reached_Place-1][Character_Nr] - 1;
            LcdDataWrite(Name_HL[Reached_Place-1][Character_Nr]); Set_LCD_Cursur(1, 11+Character_Nr); 
          }
          if (gamekit.button_pressed(butt_LEFT)) {
            Name_HL[Reached_Place-1][Character_Nr] = Name_HL[Reached_Place-1][Character_Nr] + 1;
            LcdDataWrite(Name_HL[Reached_Place-1][Character_Nr]); Set_LCD_Cursur(1, 11+Character_Nr); 
          }
          if (gamekit.button_pressed(butt_DOWN)) {
            Character_Nr++;
            if (Character_Nr > 4) { Character_Nr = 0; }
            Set_LCD_Cursur(1, 11+Character_Nr);
          }
          if (gamekit.button_pressed(butt_UP)) {
            Character_Nr--;
            if (Character_Nr < 0) { Character_Nr = 4; }
            Set_LCD_Cursur(1, 11+Character_Nr);
          }

        } while ( (gamekit.button_pressed(butt_FUNCA)==false) && (gamekit.button_pressed(butt_FUNCB)==false) );
        Save_Settings();
      }
      
      Clear_LCD();
      while (true) {   //Show Hitlist
        Set_LCD_Cursur(0, 0); LcdDataWrite('H'); LcdDataWrite('i'); LcdDataWrite('g'); LcdDataWrite('h'); LcdDataWrite('s'); LcdDataWrite('c'); LcdDataWrite('o'); LcdDataWrite('r'); LcdDataWrite('e'); 
        Set_LCD_Cursur(1, 0); LcdDataWrite('1'); LcdDataWrite('.'); LcdDataWrite(Name_HL[0][0]); LcdDataWrite(Name_HL[0][1]); LcdDataWrite(Name_HL[0][2]); LcdDataWrite(Name_HL[0][3]); LcdDataWrite(Name_HL[0][4]); LcdDataWrite(' '); LcdNumberWrite(Points_HL[0],5); LcdDataWrite(','); LcdNumberWrite(Level_HL[0],2); 
        delay(3000); Clear_LCD();
        Set_LCD_Cursur(0, 0); LcdDataWrite('1'); LcdDataWrite('.'); LcdDataWrite(Name_HL[0][0]); LcdDataWrite(Name_HL[0][1]); LcdDataWrite(Name_HL[0][2]); LcdDataWrite(Name_HL[0][3]); LcdDataWrite(Name_HL[0][4]); LcdDataWrite(' '); LcdNumberWrite(Points_HL[0],5); LcdDataWrite(','); LcdNumberWrite(Level_HL[0],2); 
        Set_LCD_Cursur(1, 0); LcdDataWrite('2'); LcdDataWrite('.'); LcdDataWrite(Name_HL[1][0]); LcdDataWrite(Name_HL[1][1]); LcdDataWrite(Name_HL[1][2]); LcdDataWrite(Name_HL[1][3]); LcdDataWrite(Name_HL[1][4]); LcdDataWrite(' '); LcdNumberWrite(Points_HL[1],5); LcdDataWrite(','); LcdNumberWrite(Level_HL[1],2); 
        delay(3000); Clear_LCD();
        Set_LCD_Cursur(0, 0); LcdDataWrite('2'); LcdDataWrite('.'); LcdDataWrite(Name_HL[1][0]); LcdDataWrite(Name_HL[1][1]); LcdDataWrite(Name_HL[1][2]); LcdDataWrite(Name_HL[1][3]); LcdDataWrite(Name_HL[1][4]); LcdDataWrite(' '); LcdNumberWrite(Points_HL[1],5); LcdDataWrite(','); LcdNumberWrite(Level_HL[1],2); 
        Set_LCD_Cursur(1, 0); LcdDataWrite('3'); LcdDataWrite('.'); LcdDataWrite(Name_HL[2][0]); LcdDataWrite(Name_HL[2][1]); LcdDataWrite(Name_HL[2][2]); LcdDataWrite(Name_HL[2][3]); LcdDataWrite(Name_HL[2][4]); LcdDataWrite(' '); LcdNumberWrite(Points_HL[2],5); LcdDataWrite(','); LcdNumberWrite(Level_HL[2],2); 
        delay(3000); Clear_LCD();
      }
    }
  }
}


void Menu() {
  for (int Counter_x=0; Counter_x<5; Counter_x++) {     //Clear screen
    for (int Counter_y=0; Counter_y<7; Counter_y++) { 
      PutPixel(Counter_x, Counter_y, 0);
    }
  }
  PutPixel(1,0,15);
  PutPixel(2,0,15);
  PutPixel(3,0,15);
  PutPixel(1,1,15);
  PutPixel(3,1,15);
  PutPixel(1,2,15);
  PutPixel(2,2,15);
  PutPixel(3,2,15);
  PutPixel(1,3,15);
  PutPixel(1,4,15);
  
  delay(2000);
  
  
  for (int Counter=0; Counter<3; Counter++) {  
    if (sound_volume >= Counter) { 
      PutPixel(Counter, 6, 15); 
    } else {
      PutPixel(Counter, 6, 0); 
    }
  }

  do {
    Play_Melody();
    if ( (gamekit.button_pressed(butt_UP)) || (gamekit.button_pressed(butt_DOWN)) || (gamekit.button_pressed(butt_LEFT)) || (gamekit.button_pressed(butt_RIGHT)) ) {
      sound_volume++;
      if (sound_volume>2) { 
        sound_volume=0;  
      }
      gamekit.play_tone(400, 1, sound_volume-1);
      for (int Counter=0; Counter<3; Counter++) {
        if (sound_volume >= Counter) { 
          PutPixel(Counter, 6, 15); 
        } else {
          PutPixel(Counter, 6, 0); 
        }
      }
    }
    
  } while ( (gamekit.button_pressed(butt_FUNCA)==false) && (gamekit.button_pressed(butt_FUNCB)==false) );
  Save_Settings();
  
  for (int Counter_x=0; Counter_x<5; Counter_x++) {  //Rewrite Screen
    for (int Counter_y=0; Counter_y<7; Counter_y++) { 
      PutPixel(Counter_x,Counter_y,color_fixed*Field[Counter_x][Counter_y]);
      PaintStone(stone_color);
    }
  }
}

void Save_Settings() {
  EEPROM.write(106,123);   //Testnumber if date in eeprom is availible
  EEPROM.write(107,sound_volume);

  EEPROMWriteInt(108,Points_HL[0]);
  EEPROMWriteInt(110,Points_HL[1]);
  EEPROMWriteInt(112,Points_HL[2]);

  EEPROM.write(114,Level_HL[0]);
  EEPROM.write(115,Level_HL[1]);
  EEPROM.write(116,Level_HL[2]);

  EEPROM.write(117,Name_HL[0][0]);
  EEPROM.write(118,Name_HL[0][1]);
  EEPROM.write(119,Name_HL[0][2]);
  EEPROM.write(120,Name_HL[0][3]);
  EEPROM.write(121,Name_HL[0][4]);

  EEPROM.write(122,Name_HL[1][0]);
  EEPROM.write(123,Name_HL[1][1]);
  EEPROM.write(124,Name_HL[1][2]);
  EEPROM.write(125,Name_HL[1][3]);
  EEPROM.write(126,Name_HL[1][4]);

  EEPROM.write(127,Name_HL[2][0]);
  EEPROM.write(128,Name_HL[2][1]);
  EEPROM.write(129,Name_HL[2][2]);
  EEPROM.write(130,Name_HL[2][3]);
  EEPROM.write(131,Name_HL[2][4]); 
}

void Load_Data() {
  if (EEPROM.read(106)==123) {
    sound_volume = EEPROM.read(107);
    
    Points_HL[0] = EEPROMReadInt(108);
    Points_HL[1] = EEPROMReadInt(110);
    Points_HL[2] = EEPROMReadInt(112);
    
    Level_HL[0] = EEPROM.read(114);
    Level_HL[1] = EEPROM.read(115);
    Level_HL[2] = EEPROM.read(116);
    
    Name_HL[0][0] = EEPROM.read(117);
    Name_HL[0][1] = EEPROM.read(118);
    Name_HL[0][2] = EEPROM.read(119);
    Name_HL[0][3] = EEPROM.read(120);
    Name_HL[0][4] = EEPROM.read(121);

    Name_HL[1][0] = EEPROM.read(122);
    Name_HL[1][1] = EEPROM.read(123);
    Name_HL[1][2] = EEPROM.read(124);
    Name_HL[1][3] = EEPROM.read(125);
    Name_HL[1][4] = EEPROM.read(126);

    Name_HL[2][0] = EEPROM.read(127);
    Name_HL[2][1] = EEPROM.read(128);
    Name_HL[2][2] = EEPROM.read(129);
    Name_HL[2][3] = EEPROM.read(130);
    Name_HL[2][4] = EEPROM.read(131);
    
  } else {
    sound_volume = 1;
 
    Name_HL[0][0] = 'M';
    Name_HL[0][1] = 'a';
    Name_HL[0][2] = 't';
    Name_HL[0][3] = 'z';
    Name_HL[0][4] = 'e';
 
    Name_HL[1][0] = 'P';
    Name_HL[1][1] = 'e';
    Name_HL[1][2] = 't';
    Name_HL[1][3] = 'e';
    Name_HL[1][4] = 'r';
 
    Name_HL[2][0] = 'P';
    Name_HL[2][1] = 'a';
    Name_HL[2][2] = 'u';
    Name_HL[2][3] = 'l';
    Name_HL[2][4] = ' ';
 
    Points_HL[0] = 3500;
    Points_HL[1] = 1500;
    Points_HL[2] = 500;

    Level_HL[0] = 5;
    Level_HL[1] = 2;
    Level_HL[2] = 1;
  }  
  gamekit.play_tone(400, 1, sound_volume-1);
}

void Read_Serial_In() {
  if ( (Serial.available()>0) && (serialdata==-99) ) {
   serialdata = Serial.read(); 
  } 
 
  if (serialdata=='f') {
    Clear_LCD();
    Set_LCD_Cursur(0, 0); LcdDataWrite('C'); LcdDataWrite('o'); LcdDataWrite('n'); LcdDataWrite('g'); LcdDataWrite('r'); LcdDataWrite('a'); LcdDataWrite('t'); LcdDataWrite('u'); LcdDataWrite('l'); LcdDataWrite('a'); LcdDataWrite('t'); LcdDataWrite('i'); LcdDataWrite('o'); LcdDataWrite('n'); LcdDataWrite('s');
    Set_LCD_Cursur(1, 0); LcdDataWrite('Y'); LcdDataWrite('o'); LcdDataWrite('u'); LcdDataWrite(' '); LcdDataWrite('W'); LcdDataWrite('i'); LcdDataWrite('n'); LcdDataWrite('!');
    Set_LCD_Cursur(2, 0);     while (true) {
      for (int Counter_x=0; Counter_x<5; Counter_x++) {
        for (int Counter_y=0; Counter_y<7; Counter_y++) {
          PutPixel(Counter_x,Counter_y,15);
          delay(30);
        }
      }
      for (int Counter_x=0; Counter_x<5; Counter_x++) {
        for (int Counter_y=0; Counter_y<7; Counter_y++) {
          PutPixel(Counter_x,Counter_y,0);
          delay(30);
        }
      }
    }
  }
}








void loop() {
  
  //Scan for second player
  Set_LCD_Cursur(0, 0); LcdDataWrite('S'); LcdDataWrite('c'); LcdDataWrite('a'); LcdDataWrite('n'); LcdDataWrite('n'); LcdDataWrite('i'); LcdDataWrite('n'); LcdDataWrite('g'); LcdDataWrite(' '); LcdDataWrite('f'); LcdDataWrite('o'); LcdDataWrite('r');  
  Set_LCD_Cursur(1, 0); LcdDataWrite('s'); LcdDataWrite('e'); LcdDataWrite('c'); LcdDataWrite('o'); LcdDataWrite('n'); LcdDataWrite('d'); LcdDataWrite(' '); LcdDataWrite('P'); LcdDataWrite('l'); LcdDataWrite('a'); LcdDataWrite('y'); LcdDataWrite('e'); LcdDataWrite('r'); 
  for (int Counter_x=0; Counter_x<5; Counter_x++) {
    for (int Counter_y=0; Counter_y<7; Counter_y++) {
      PutPixel(Counter_x,Counter_y,15);
      if (Serial.available()>0) {
        serialdata = Serial.read(); 
        if (serialdata=='a') {        //Other player found
          SinglePlayer = false; 
          Serial.write('b');          //"b" = "Yes, I am here. Let's start the game." 
        }
        if (serialdata=='b') {
          SinglePlayer = false;
        }
        Counter_x = 99; 
        Counter_y = 99;
      } else {
        Serial.write('a');            //"a" = "Is someone there?"
      }
      delay(50);
    }
  }
  Clear_LCD(); //Clear LCD & screen
  for (int Counter_x=0; Counter_x<5; Counter_x++) {
    for (int Counter_y=0; Counter_y<7; Counter_y++) {
      PutPixel(Counter_x,Counter_y,0);
    }
  }
  if (SinglePlayer==false) {
    Set_LCD_Cursur(0, 0); LcdDataWrite('E'); LcdDataWrite('n'); LcdDataWrite('t'); LcdDataWrite('e'); LcdDataWrite('r'); LcdDataWrite('i'); LcdDataWrite('n'); LcdDataWrite('g'); 
    Set_LCD_Cursur(1, 0); LcdDataWrite('T'); LcdDataWrite('w'); LcdDataWrite('o'); LcdDataWrite('-'); LcdDataWrite('P'); LcdDataWrite('l'); LcdDataWrite('a'); LcdDataWrite('y'); LcdDataWrite('e'); LcdDataWrite('r'); LcdDataWrite('-'); LcdDataWrite('M'); LcdDataWrite('o'); LcdDataWrite('d'); LcdDataWrite('e'); 
    sound_volume = 0;
    delay(1500);
    Serial.read();      //Clear Serial IN if an "a" is still in
    serialdata = -99;
  } else {
    Set_LCD_Cursur(0, 0); LcdDataWrite('E'); LcdDataWrite('n'); LcdDataWrite('t'); LcdDataWrite('e'); LcdDataWrite('r'); LcdDataWrite('i'); LcdDataWrite('n'); LcdDataWrite('g'); 
    Set_LCD_Cursur(1, 0); LcdDataWrite('O'); LcdDataWrite('n'); LcdDataWrite('e'); LcdDataWrite('-'); LcdDataWrite('P'); LcdDataWrite('l'); LcdDataWrite('a'); LcdDataWrite('y'); LcdDataWrite('e'); LcdDataWrite('r'); LcdDataWrite('-'); LcdDataWrite('M'); LcdDataWrite('o'); LcdDataWrite('d'); LcdDataWrite('e'); 
    Load_Data();
    delay(1500);
  } 
  
  
  Clear_LCD();
  LCD_Write_Points();

  while (true) {    //Main Game Routine  
    ReadKeys();
    Play_Melody();
    if (SinglePlayer==false) {
      Read_Serial_In();
    }
    if (millis() > timer+Millis_per_Step) {
      if (Wait_On_Top < Wait_On_Top_Limit) {
        stone_color = color_top;
        PaintStone(stone_color);
        Wait_On_Top++;
        timer = millis();
      } else {
        stone_color = color_normal;
        Move_Stone_One_Down();
      }
    }
  }
  
  
} 


















// ----------------------------------------------
//                LCD Control
// ----------------------------------------------

// the Qx in the order they are connected on the chip
int DI = 1;
int RW = 2;
int Enable = 3;
int DB[] = {
  7, 6, 5, 4};

void LcdInit() {
  
  pinMode(Dout,OUTPUT);
  pinMode(STR,OUTPUT);
  pinMode(CLK,OUTPUT);

  delay(100);
  // initialize LCD after a short pause
  // needed by the LCD's controller

  /////////// 4 pin initialization
  LcdCommandWrite(0x03); // function set:
  // 4 pin initialization
  delay(64);
  LcdCommandWrite(0x03); // function set:
  // 4 pin initialization
  delay(50);
  LcdCommandWrite(0x03); // function set:
  // 4 pin initialization
  delay(50);
  LcdCommandWrite(0x02); // function set:
  // 4 pin initialization
  delay(50);
  LcdCommandWrite(0x2C); // function set:
  // 4-bit interface, 1 display lines, 5x7 font
  /////////// end of 4 pin initialization 

  delay(20);
  LcdCommandWrite(0x06); // entry mode set:
  // increment automatically, no display shift
  delay(20);
  LcdCommandWrite(0x0E); // display control:
  // turn display on, cursor on, no blinking
  delay(20);
  // clear display, set cursor position to zero
  LcdCommandWrite(0x01); 
  delay(100);

  LcdCommandWrite(0x80); // display control:
  delay(20);
}

void sendByteOut(int value) {
  shiftOut(Dout, CLK, LSBFIRST, value);
  digitalWrite(STR, HIGH);
  delayMicroseconds(10);
  digitalWrite(STR,LOW);
}

void LcdCommandWrite(int value) {
  int i = 0;
  int value1 = 0;
  int control = 0; // stores DI and RW
  digitalWrite(STR,LOW); // set the strobe LOW
  control = value >> 8; // get the control signals DI and RW
  control <<= 5; // shift the control signals to the left
  value1 = value;
  value1 >>= 4; //send the first 4 databits (from 8) 
  value1 |= control; // set the control values
  value1 &= 239; // set Enable LOW
  sendByteOut(value1);
  value1 |= 16; // Set Enable HIGH
  sendByteOut(value1);
  value1 &= 239; // set Enable LOW
  sendByteOut(value1);

  delay(1);

  value &= 15; // set HByte to zero 
  value |= control; // set the control values
  value &= 239; // set Enable LOW
  sendByteOut(value);
  value |= 16; // Set Enable HIGH
  sendByteOut(value);
  value &= 239; // set Enable LOW
  sendByteOut(value);
}

void LcdDataWrite(int value) {
  int i = 0;
  int value1 = 0;
  digitalWrite(STR,LOW); // set the strobe LOW
  value1 =value;
  value1 >>= 4; //send the first 4 databits (from 8) 
  value1 |= 64; // set DI HIGH
  value1 &= 223; // set RW LOW
  value1 &= 239; // set Enable LOW
  sendByteOut(value1);
  value1 |= 16; // Set Enable HIGH
  sendByteOut(value1);
  value1 &= 239; // set Enable LOW
  sendByteOut(value1);

  delay(1);

  value &= 15; // set HByte to zero 
  value |= 64; // set DI HIGH
  value &= 223; // set RW LOW
  value &= 239; // set Enable LOW
  sendByteOut(value);
  value |= 16; // Set Enable HIGH
  sendByteOut(value);
  value &= 239; // set Enable LOW
  sendByteOut(value);
}

// efficient way of multiplying numbers by themselves
int pow(int base, int expo) {
  int temp = 1;
  for (int c = 1; c <= expo; c++) {
    temp *= base;
  }
  return temp;
}

// checks out how many digits there are in a number
int estimateDigits(int nr) {
  int dec = 10;
  int temp = 1;
  int div = nr/dec;
  while (div > 0) {
    dec *= 10;
    div = nr/dec;
    temp++;
  }
  return temp;
}

// shows numbers on the display
void LcdNumberWrite(int nr) {
  int digits = estimateDigits(nr);
  LcdNumberWrite(nr, digits);
}

// this function help us to write numbers 
// with more than one digit
void LcdNumberWrite(int nr, int digits) {
  boolean Nothing_Printed_SoFare = true;
  for (int i = digits-1; i >= 0; i--) {
    int dec = pow(10,i);
    int div = nr/dec;
    if (div > 0) {
      nr -= div*dec; 
      LcdDataWrite(div+48); 
      Nothing_Printed_SoFare = false;
    } else if ( (Nothing_Printed_SoFare==false) || (i==0)) {
      LcdDataWrite(48); 
    } else if ((Nothing_Printed_SoFare==true) && (i!=0)) {
      LcdDataWrite(' '); 
    }
  }
}

void Set_LCD_Cursur(int row, int col) {
  int row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };
  unsigned char setCursor = 0x80 | (col + row_offsets[row]);
  LcdCommandWrite(setCursor); // shift cursor to the left  
}

void Clear_LCD() {
  Set_LCD_Cursur(0,0);
  for (int Counter=0; Counter<24; Counter++) {
     LcdDataWrite(' '); 
  }
  Set_LCD_Cursur(1,0);
  for (int Counter=0; Counter<24; Counter++) {
     LcdDataWrite(' '); 
  }
}





// EEPROM Functions for Integer
void EEPROMWriteInt(int p_address, int p_value)
	{
	byte lowByte = ((p_value >> 0) & 0xFF);
	byte highByte = ((p_value >> 8) & 0xFF);

	EEPROM.write(p_address, lowByte);
	EEPROM.write(p_address + 1, highByte);
	}

unsigned int EEPROMReadInt(int p_address)
	{
	byte lowByte = EEPROM.read(p_address);
	byte highByte = EEPROM.read(p_address + 1);

	return ((lowByte << 0) & 0xFF) + ((highByte << 8) & 0xFF00);
	}


